/*
 * (C) COPYRIGHT 2014 CRZ
 *
 * File Name : test_sensor_HTU21.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 07/30/2014
 */

/* includes */

#include "hw_config.h"
#include "global_types.h"
#include "uart_queue.h"

/* functions */

bool UartQueue_Serial_Is_Empty(void);
uint16_t UartQueue_Serial_DeQueue(void);

/*
 * device operation
 */

static bool g_print_menu_control_flag = TRUE;
static bool g_print_menu_control_flag_2 = TRUE;

static void print_menu_command(void)
{
    if(TRUE == g_print_menu_control_flag)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("1> Output port 0 On\n");
        printf("2> Output port 0 Off\n");
        printf("3> Output port 1 On\n");
        printf("4> Output port 1 Off\n");
        printf("5> Output port 2 On\n");
        printf("6> Output port 2 Off\n");
        printf("7> Output port 3 On\n");
        printf("8> Output port 4 Off\n");
        printf("---------------------\n");
        printf("9> Read Input ports\n");
        printf("a> Read DIP switch ports\n");
        printf("b> Read Output ports\n");
        printf("---------------------\n");
        printf("x> quit\n\n");
    }

    g_print_menu_control_flag = FALSE;
}

static void print_menu_command_2(void)
{
    if(TRUE == g_print_menu_control_flag_2)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("S> Read port State\n");
        printf("0> Output port 1 Off\n");
        printf("1> Output port 1 On\n");
        printf("2> Output port 1 Blink\n");
        printf("3> Output port 2 Off\n");
        printf("4> Output port 2 On\n");
        printf("5> Output port 2 Blink\n");
        printf("6> Output port 3 Off\n");
        printf("7> Output port 3 On\n");
        printf("8> Output port 3 Blink\n");
        printf("9> Output port 4 Off\n");
        printf("A> Output port 4 On\n");
        printf("B> Output port 4 Blink\n");
        printf("---------------------\n");
        printf("x> quit\n\n");
    }

    g_print_menu_control_flag_2 = FALSE;
}

void Actuator_Port_Initialization(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    /* DIP Switch setup - Input */

    GPIO_InitStructure.GPIO_Pin   = GPIO_MCU_MIO_0_PIN | GPIO_MCU_MIO_1_PIN | GPIO_MCU_MIO_3_PIN;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
    GPIO_Init(GPIO_MCU_MIO, &GPIO_InitStructure);

    /* PA setup - Input */

    GPIO_InitStructure.GPIO_Pin   = GPIO_MCU_PA_0_PIN | GPIO_MCU_PA_1_PIN | GPIO_MCU_PA_2_PIN | GPIO_MCU_PA_3_PIN;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
    GPIO_Init(GPIO_MCU_PA, &GPIO_InitStructure);

    /* PA setup - Output */

    GPIO_InitStructure.GPIO_Pin   = GPIO_MCU_PA_4_PIN | GPIO_MCU_PA_5_PIN | GPIO_MCU_PA_6_PIN | GPIO_MCU_PA_7_PIN;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
    GPIO_Init(GPIO_MCU_PA, &GPIO_InitStructure);

    GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN | GPIO_MCU_PA_5_PIN | GPIO_MCU_PA_6_PIN | GPIO_MCU_PA_7_PIN);
}

uint8_t Actuator_Output_Port_1_GetValue(void)
{
    return GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
}

void Actuator_Output_Port_1_On(void)
{
    GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
}

void Actuator_Output_Port_1_Off(void)
{
    GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
}

void Actuator_Output_Port_1_Toggle(void)
{
    GPIO_WriteBit(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN,
                  (BitAction)(1 - Actuator_Output_Port_1_GetValue()));
}

uint8_t Actuator_Output_Port_2_GetValue(void)
{
    return GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_5_PIN);
}

void Actuator_Output_Port_2_On(void)
{
    GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_5_PIN);
}

void Actuator_Output_Port_2_Off(void)
{
    GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_5_PIN);
}

void Actuator_Output_Port_2_Toggle(void)
{
    GPIO_WriteBit(GPIO_MCU_PA, GPIO_MCU_PA_5_PIN,
                  (BitAction)(1 - Actuator_Output_Port_2_GetValue()));
}

uint8_t Actuator_Output_Port_3_GetValue(void)
{
    return GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN);
}

void Actuator_Output_Port_3_On(void)
{
    GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN);
}

void Actuator_Output_Port_3_Off(void)
{
    GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN);
}

void Actuator_Output_Port_3_Toggle(void)
{
    GPIO_WriteBit(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN,
                  (BitAction)(1 - Actuator_Output_Port_3_GetValue()));
}

uint8_t Actuator_Output_Port_4_GetValue(void)
{
    return GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_7_PIN);
}

void Actuator_Output_Port_4_On(void)
{
    GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_7_PIN);
}

void Actuator_Output_Port_4_Off(void)
{
    GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_7_PIN);
}

void Actuator_Output_Port_4_Toggle(void)
{
    GPIO_WriteBit(GPIO_MCU_PA, GPIO_MCU_PA_7_PIN,
                  (BitAction)(1 - Actuator_Output_Port_4_GetValue()));
}

void Test_Actuator(void)
{
    uint8_t ch;

    DEBUG_MSG_FUNC_START;

    while(1)
    {
        print_menu_command();

        if(FALSE == UartQueue_Serial_Is_Empty())
        {
            ch = UartQueue_Serial_DeQueue();
            printf("%c is selected\n\n", (char)ch);

            switch((char)ch)
            {
            case '1': Actuator_Output_Port_1_On();   break;
            case '2': Actuator_Output_Port_1_Off(); break;
            case '3': GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_5_PIN);   break;
            case '4': GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_5_PIN); break;
            case '5': GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN);   break;
            case '6': GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN); break;
            case '7': GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_7_PIN);   break;
            case '8': GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_7_PIN); break;

            case '9':
                printf("Input Port P0: %d, P1: %d, P2: %d, P3: %d\n",
                    GPIO_ReadInputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_0_PIN),
                    GPIO_ReadInputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_1_PIN),
                    GPIO_ReadInputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_2_PIN),
                    GPIO_ReadInputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_3_PIN));
                break;

            case 'a':
                printf("Dip Switch Port D4: %d, D5: %d, D6: %d\n",
                    GPIO_ReadInputDataBit(GPIO_MCU_MIO, GPIO_MCU_MIO_0_PIN),
                    GPIO_ReadInputDataBit(GPIO_MCU_MIO, GPIO_MCU_MIO_1_PIN),
                    GPIO_ReadInputDataBit(GPIO_MCU_MIO, GPIO_MCU_MIO_3_PIN));
                break;

            case 'b':
                printf("Output Port P0: %d, P1: %d, P2: %d, P3: %d\n",
                    GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN),
                    GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_5_PIN),
                    GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN),
                    GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_7_PIN));
                break;
            }

            g_print_menu_control_flag = TRUE;

            if('x' == (char)ch)
            {
                break;
            }
        }
    }

    return;
}

typedef enum
{
    MCU_COMM_RX_STATE_INIT               = 0,
    MCU_COMM_RX_STATE_CONTROL_T_RECEIVED = 1,
    MCU_COMM_RX_STATE_CONTROL_W_RECEIVED = 2,
    MCU_COMM_RX_STATE_CONTROL_L_RECEIVED = 3,
    MCU_COMM_RX_STATE_REQ_STS_RECEIVED   = 4,
} Mcu_Comm_Rx_State_E;

Mcu_Comm_Rx_State_E g_rx_curr_state = MCU_COMM_RX_STATE_INIT;

bool g_actuator_blink_state[ACTUATOR_OUT_PORT_NUM] = {FALSE, };

void control_state_zigbee_comm(uint8_t ch)
{
    switch(g_rx_curr_state)
    {
    case MCU_COMM_RX_STATE_INIT:
        if('T' == (char)ch)
        {
            g_rx_curr_state = MCU_COMM_RX_STATE_CONTROL_T_RECEIVED;
        }
        break;

    case MCU_COMM_RX_STATE_CONTROL_T_RECEIVED:
        if('W' == (char)ch)
        {
            g_rx_curr_state = MCU_COMM_RX_STATE_CONTROL_W_RECEIVED;
        }
        else
        {
            g_rx_curr_state = MCU_COMM_RX_STATE_INIT;
        }
        break;

    case MCU_COMM_RX_STATE_CONTROL_W_RECEIVED:
        if('L' == (char)ch)
        {
            g_rx_curr_state = MCU_COMM_RX_STATE_CONTROL_L_RECEIVED;
        }
        else
        {
            g_rx_curr_state = MCU_COMM_RX_STATE_INIT;
        }
        break;

    case MCU_COMM_RX_STATE_CONTROL_L_RECEIVED:
        g_rx_curr_state = MCU_COMM_RX_STATE_INIT;
        break;

    case MCU_COMM_RX_STATE_REQ_STS_RECEIVED:
        break;
    default:
        break;
    }
}

void Test_Actuator_ZigBee_Comm_TX(void)
{
    uint8_t ch;

    DEBUG_MSG_FUNC_START;

    while(1)
    {
        print_menu_command_2();

        if(FALSE == UartQueue_Comm_Is_Empty())
        {
            ch = UartQueue_Comm_DeQueue();
            printf("0x%02X is received\n\n", ch);
        }

        if(FALSE == UartQueue_Serial_Is_Empty())
        {
            ch = UartQueue_Serial_DeQueue();
            printf("%c is selected\n\n", (char)ch);

            switch((char)ch)
            {
            case 'S': case 's': /* State */
                UartComm_SendData_OneChar('S');
                break;

            /* Port #1 */
            case '0': /* Off */
                UartComm_SendData_OneChar('0');
                break;

            case '1': /* On */
                UartComm_SendData_OneChar('1');
                break;

            case '2': /* Blink */
                UartComm_SendData_OneChar('2');
                break;

            /* Port #2 */
            case '3': /* Off */
                UartComm_SendData_OneChar('3');
                break;

            case '4': /* On */
                UartComm_SendData_OneChar('4');
                break;

            case '5': /* Blink */
                UartComm_SendData_OneChar('5');
                break;

            /* Port #3 */
            case '6': /* Off */
                UartComm_SendData_OneChar('6');
                break;

            case '7': /* On */
                UartComm_SendData_OneChar('7');
                break;

            case '8': /* Blink */
                UartComm_SendData_OneChar('8');
                break;

            /* Port #4 */
            case '9': /* Off */
                UartComm_SendData_OneChar('9');
                break;

            case 'A': case 'a': /* On */
                UartComm_SendData_OneChar('A');
                break;

            case 'B': case 'b': /* Blink */
                UartComm_SendData_OneChar('B');
                break;
            }

            g_print_menu_control_flag_2 = TRUE;

            if('x' == (char)ch)
            {
                break;
            }
        }
    }

    return;
}

void Test_Actuator_ZigBee_Comm_RX(void)
{
    uint8_t ch;

    if(FALSE == UartQueue_Comm_Is_Empty())
    {
        ch = UartQueue_Comm_DeQueue();
        printf("%c is received\n\n", (char)ch);

        switch((char)ch)
        {
        case 'S':
            {
                uint8_t ch_sendData, ch_1_data, ch_2_data, ch_3_data, ch_4_data;

                if(TRUE == g_actuator_blink_state[0])
                {
                    ch_1_data = 2;
                }
                else
                {
                    ch_1_data = Actuator_Output_Port_1_GetValue();
                }

                if(TRUE == g_actuator_blink_state[1])
                {
                    ch_2_data = 2;
                }
                else
                {
                    ch_2_data = Actuator_Output_Port_2_GetValue();
                }

                if(TRUE == g_actuator_blink_state[2])
                {
                    ch_3_data = 2;
                }
                else
                {
                    ch_3_data = Actuator_Output_Port_3_GetValue();
                }

                if(TRUE == g_actuator_blink_state[3])
                {
                    ch_4_data = 2;
                }
                else
                {
                    ch_4_data = Actuator_Output_Port_4_GetValue();
                }

                ch_sendData = (ch_4_data << 6) | (ch_3_data << 4) | (ch_2_data << 2) | ch_1_data;

                UartComm_SendData_OneChar(ch_sendData);
                UartComm_SendData_OneChar(0);
                UartComm_SendData_OneChar(0);

                g_rx_curr_state = MCU_COMM_RX_STATE_INIT;
            }
            break;

        /* Port #1 */
        case '0': /* Off */
            g_actuator_blink_state[0] = FALSE;
            Actuator_Output_Port_1_Off();
            break;

        case '1': /* On */
            g_actuator_blink_state[0] = FALSE;
            Actuator_Output_Port_1_On();
            break;

        case '2': /* Blink */
            g_actuator_blink_state[0] = TRUE;
            break;

        /* Port #2 */
        case '3': /* Off */
            g_actuator_blink_state[1] = FALSE;
            Actuator_Output_Port_2_Off();
            break;

        case '4': /* On */
            g_actuator_blink_state[1] = FALSE;
            Actuator_Output_Port_2_On();
            break;

        case '5': /* Blink */
            g_actuator_blink_state[1] = TRUE;
            break;

        /* Port #3 */
        case '6': /* Off */
            g_actuator_blink_state[2] = FALSE;
            Actuator_Output_Port_3_Off();
            break;

        case '7': /* On */
            g_actuator_blink_state[2] = FALSE;
            Actuator_Output_Port_3_On();
            break;

        case '8': /* Blink */
            g_actuator_blink_state[2] = TRUE;
            break;

        /* Port #4 */
        case '9': /* Off */
            g_actuator_blink_state[3] = FALSE;
            Actuator_Output_Port_4_Off();
            break;

        case 'A': /* On */
            g_actuator_blink_state[3] = FALSE;
            Actuator_Output_Port_4_On();
            break;

        case 'B': /* Blink */
            g_actuator_blink_state[3] = TRUE;
            break;
        }

//            control_state_zigbee_comm(ch);
    }
}

