/*
 * (C) COPYRIGHT 2014 CRZ
 *
 * File Name : test_sensor_HTU21.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 07/30/2014
 */

/* includes */

#include "hw_config.h"
#include "global_types.h"

/* functions */

bool UartQueue_Serial_Is_Empty(void);
uint16_t UartQueue_Serial_DeQueue(void);

#define TEMP_MEASURE_HOLD      0xE3
#define HUMID_MEASURE_HOLD     0xE5
#define TEMP_MEASURE_NO_HOLD   0xF3
#define HUMID_MEASURE_NO_HOLD  0xF5

#define SOFT_RESET             0xFE

extern uint8_t Buffer_Rx1[];
extern uint8_t Buffer_Tx1[];

#define HTU21_SLAVE_ADDRESS     0x40 /* Slave Address : "100 0000" */

/*
 * device operation
 */

uint8_t SENSOR_HTU21_Read_Bytes(uint8_t devAddr, uint8_t regAddr, uint8_t length, uint8_t *data)
{
    /*
     * devAddr should be shifted 1 bit to left ...
     */
    
    Status  i2cRW_status;

    /* Disable FSMC */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, DISABLE);

    Buffer_Tx1[0] = regAddr;

    i2cRW_status = I2C_Master_BufferWrite(I2C1, Buffer_Tx1, 1, Interrupt, devAddr << 1);
    if(Error == i2cRW_status)
    {
        DEBUG_MSG_FUNC_ERROR;
        return 0;
    }

    i2cRW_status = I2C_Master_BufferRead(I2C1, data, length, Polling, devAddr << 1);
    if(Error == i2cRW_status)
    {
        DEBUG_MSG_FUNC_ERROR;
        return 0;
    }

    /* Enable FSMC */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    return length;
}

int8_t SENSOR_HTU21_Write_Bytes(uint8_t devAddr, uint8_t regAddr, uint8_t length, uint8_t *data)
{
    Status  i2cRW_status;

    /* Disable FSMC */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, DISABLE);

    Buffer_Tx1[0] = regAddr;
    memcpy(&Buffer_Tx1[1], data, length);

    i2cRW_status = I2C_Master_BufferWrite(I2C1, Buffer_Tx1, length + 1, Interrupt, devAddr << 1);
    if(Error == i2cRW_status)
    {
        DEBUG_MSG_FUNC_ERROR;
        return -1;
    }

    /* Enable FSMC */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    return 0;
}

int8_t SENSOR_HTU21_Write_Command(uint8_t devAddr, uint8_t data)
{
    Status  i2cRW_status;

    /* Disable FSMC */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, DISABLE);

    Buffer_Tx1[0] = data;

    i2cRW_status = I2C_Master_BufferWrite(I2C1, Buffer_Tx1, 1, Interrupt, devAddr << 1);
    if(Error == i2cRW_status)
    {
        DEBUG_MSG_FUNC_ERROR;
        return -1;
    }

    /* Enable FSMC */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    return 0;
}

static bool g_print_menu_control_flag = TRUE;

static void print_menu_command(void)
{
    if(TRUE == g_print_menu_control_flag)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("1> Get Temperature Data\n");
        printf("2> Get Humidity Data\n");
        printf("---------------------\n");
        printf("x> quit\n\n");
    }

    g_print_menu_control_flag = FALSE;
}

void Test_Sensor_HTU21(void)
{
    uint8_t ch;
    uint8_t dataVal[3];

    uint16_t data16Val;

    float calcData;

    DEBUG_MSG_FUNC_START;

    Delay(300);
    SENSOR_HTU21_Write_Command(HTU21_SLAVE_ADDRESS, SOFT_RESET);
    Delay(300);

#if 0
    while(1)
    {
        SENSOR_HTU21_Read_Bytes(HTU21_SLAVE_ADDRESS, TEMP_MEASURE_HOLD, 3, dataVal);

        data16Val = dataVal[0] << 8 | dataVal[1];

        calcData = -46.85 + (175.72 * ((float)data16Val / 65536));

        printf("Temperature: %f, ", calcData);

        SENSOR_HTU21_Read_Bytes(HTU21_SLAVE_ADDRESS, HUMID_MEASURE_HOLD, 3, dataVal);

        data16Val = dataVal[0] << 8 | dataVal[1];

        calcData = -6 + (125 * ((float)data16Val / 65536));

        printf("Humidity: %f\n", data16Val, calcData);

        LED_Toggle(LED1);
        LED_Toggle(LED2);
        LED_Toggle(LED3);
    }
#else
    while(1)
    {
        print_menu_command();

        if(FALSE == UartQueue_Serial_Is_Empty())
        {
            ch = UartQueue_Serial_DeQueue();
            printf("%c is selected\n\n", (char)ch);

            switch((char)ch)
            {
            case '0':
                break;

            case '1':
                SENSOR_HTU21_Read_Bytes(HTU21_SLAVE_ADDRESS, TEMP_MEASURE_HOLD, 3, dataVal);
                printf("data[0]: 0x%02X, data[1]: 0x%02X, data[2]: 0x%02X\n", dataVal[0], dataVal[1], dataVal[2]);

                data16Val = dataVal[0] << 8 | dataVal[1];

                calcData = -46.85 + (175.72 * ((float)data16Val / 65536));

                printf("data16Val: 0x%04X, Temperature: %f\n", data16Val, calcData);
                break;

            case '2':
                SENSOR_HTU21_Read_Bytes(HTU21_SLAVE_ADDRESS, HUMID_MEASURE_HOLD, 3, dataVal);
                printf("data[0]: 0x%02X, data[1]: 0x%02X, data[2]: 0x%02X\n", dataVal[0], dataVal[1], dataVal[2]);

                data16Val = dataVal[0] << 8 | dataVal[1];

                calcData = -6 + (125 * ((float)data16Val / 65536));

                printf("data16Val: 0x%04X, Humidity: %f\n", data16Val, calcData);
                break;
            }

            g_print_menu_control_flag = TRUE;

            if('x' == (char)ch)
            {
                break;
            }
        }
    }
#endif

    return;
}

