/*
 * (C) COPYRIGHT 2014 CRZ
 *
 * File Name : test_sensor_LPS25H.c
 * Author    : 
 * Version   : V1.0
 * Date      : 09/25/2014
 */

/* includes */

#include "hw_config.h"
#include "global_types.h"
#include "test_sensor_LPS25H.h"
/* functions */

bool UartQueue_Serial_Is_Empty(void);
uint16_t UartQueue_Serial_DeQueue(void);

extern uint8_t Buffer_Rx1[];
extern uint8_t Buffer_Tx1[];

/*connect the connector*/
#define LPS25H_SLAVE_ADDRESS_LOW     0x5C          /* Slave Address : "0xB8" */
/*remove the connector*/
#define LPS25H_SLAVE_ADDRESS_HIGH     0x5D         /* Slave Address : "0xBA" */

//uint8_t LPS25H_SLAVE_ADDRESS = LPS25H_SLAVE_ADDRESS_LOW;     
uint8_t LPS25H_SLAVE_ADDRESS = LPS25H_SLAVE_ADDRESS_HIGH;

uint8_t SENSOR_LPS25H_Read_Bytes(uint8_t devAddr, uint8_t regAddr, uint8_t length, uint8_t *data)
{
    /*
     * devAddr should be shifted 1 bit to left ...
     */
    
    Status  i2cRW_status;

    /* Disable FSMC */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, DISABLE);

    Buffer_Tx1[0] = regAddr;
   
    i2cRW_status = I2C_Master_BufferWrite(I2C1, Buffer_Tx1, 1, Interrupt, devAddr << 1);
    
    if(Error == i2cRW_status)
    {
        DEBUG_MSG_FUNC_ERROR;
        return 0;
    }

    i2cRW_status = I2C_Master_BufferRead(I2C1, data, length, Polling, devAddr << 1);
    if(Error == i2cRW_status)
    {
        DEBUG_MSG_FUNC_ERROR;
        return 0;
    }

    /* Enable FSMC */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    return length;
}

int8_t SENSOR_LPS25H_Write_Bytes(uint8_t devAddr, uint8_t regAddr, uint8_t length, uint8_t *data)
{
    Status  i2cRW_status;

    /* Disable FSMC */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, DISABLE);

    Buffer_Tx1[0] = regAddr;
    memcpy(&Buffer_Tx1[1], data, length);

    i2cRW_status = I2C_Master_BufferWrite(I2C1, Buffer_Tx1, length + 1, Interrupt, devAddr << 1);
    if(Error == i2cRW_status)
    {
        DEBUG_MSG_FUNC_ERROR;
        return -1;
    }

    /* Enable FSMC */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    return 0;
}

/**
 * @brief  Set the slave address according to SA0 bit.
 * @param  SA0_Bit_Status: LPS25H_SA0_LOW or LPS25H_SA0_HIGH
 * @retval None
 */
void LPS25H_Init(void)
{
    uint8_t dataVal = 0x00;
    uint8_t dataValRead = 0x00;

    //PowerOn
    dataVal|= LPS25H_MODE_ACTIVE;
    
    /* Output Data Rate selection */    
    dataVal |= LPS25H_ODR_25Hz;        //Output Data Rate: P - 25Hz, T - 25Hz 
    /* Interrupt circuit selection */
    dataVal |= LPS25H_DIFF_ENABLE;     //interrupt circuit enabled
    /* Block Data Update selection */
    dataVal |= LPS25H_BDU_READ;        //output registers not updated until MSB and LSB reading
    /* Serial Interface Mode selection */
    dataVal |= LPS25H_SPI_SIM_3W;      //3-wire interface
    
    SENSOR_LPS25H_Write_Bytes(LPS25H_SLAVE_ADDRESS, LPS25H_CTRL_REG1_ADDR, 1, &dataVal);
    SENSOR_LPS25H_Read_Bytes(LPS25H_SLAVE_ADDRESS, LPS25H_CTRL_REG1_ADDR, 1, &dataValRead);
    printf("LPS25H_CTRL_REG1_ADDR init read : 0x%02X\n", dataValRead);
   
    dataVal = 0x00;
    dataVal |= LPS25H_P_RES_AVG_512;    //LPS25H Pressure resolution selection AVG_512 
    dataVal |= LPS25H_T_RES_AVG_64;     //LPS25H Temperature resolution selection AVG_64
    
    SENSOR_LPS25H_Write_Bytes(LPS25H_SLAVE_ADDRESS, LPS25H_RES_CONF_ADDR, 1, &dataVal);
    SENSOR_LPS25H_Read_Bytes(LPS25H_SLAVE_ADDRESS, LPS25H_RES_CONF_ADDR, 1, &dataValRead);
    printf("LPS25H_RES_CONF_ADDR init read : 0x%02X\n", dataValRead);
}

static bool g_print_menu_control_flag = TRUE;

static void print_menu_command(void)
{
    if(TRUE == g_print_menu_control_flag)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("0> Get Device ID\n");
        printf("1> Get Pressure Data\n");
        printf("2> Get Temperature Data\n");
        printf("---------------------\n");
        printf("x> quit\n\n");
    }

    g_print_menu_control_flag = FALSE;
}

#define COUNT_AVERAGE_VAL 1000

void Test_Sensor_LPS25H(void)
{
    uint8_t   ch;
    uint8_t   dataVal[3];

    uint8_t   buffer[3]={0}, i;
    uint32_t  data32Val;
    int16_t   data16Val;
    uint16_t  unsigned_data16Val;

    uint32_t  count, k;
   
    float calcData;
    uint32_t  data32Val_sum;

    DEBUG_MSG_FUNC_START;

    Delay(300);
    LPS25H_Init();
    Delay(300);
    
    while(1)
    {
        print_menu_command();

        if(FALSE == UartQueue_Serial_Is_Empty())
        {
            ch = UartQueue_Serial_DeQueue();
            printf("%c is selected\n\n", (char)ch);

            switch((char)ch)
            {
            case '0':
                SENSOR_LPS25H_Read_Bytes(LPS25H_SLAVE_ADDRESS, LPS25H_WHO_AM_I_ADDR, 1, dataVal);
                printf("Device id : 0x%02X\n", dataVal[0]);  
                break;

            case '1':
                for(count = 0; count < 1000; count ++)
                {
                    data32Val = 0;
                    data32Val_sum = 0;

                    for(k = 0; k < COUNT_AVERAGE_VAL; k ++)
                    {
                        /*
                        The device is a slave with 1 byte sub-address which MSB should be 1 to enable the 
                        multiple data read/write at increasing addresses (bit 7 = 1 enables IC sub-address 
                        multi-byte auto-increment)
                        */
                        SENSOR_LPS25H_Read_Bytes(LPS25H_SLAVE_ADDRESS, LPS25H_PRESS_POUT_XL_ADDR + 0x80, 3, buffer);
                        // printf("data[0]: 0x%02X, data[1]: 0x%02X, data[2]: 0x%02X\n", buffer[0], buffer[1], buffer[2]);
                      
                        /* Build the raw data */
                        for(i = 0; i < 3; i ++)
                        {
                            data32Val |= (((uint32_t) buffer[i]) << (8 * i));
                        }
                        
                        /* convert the 2's complement 24 bit to 2's complement 32 bit */
                        if (data32Val & 0x00800000)
                        {
                            printf("data32Val & 0x00800000) is true \n");
                            data32Val |= 0xFF000000;
                        }

                        data32Val_sum += data32Val;

                        Delay(1);
                    }

                    calcData = (float)data32Val_sum / COUNT_AVERAGE_VAL / 4096.0;

                    printf("Pressure: %5.2f hPa\n", calcData);
                }
                break;

            case '2':
                data16Val=0;

                /*
                 * @brief Read LPS25H output register, and calculate the raw temperature.
                 * @param int16_t *raw_data: temperature raw value.
                 * @retval LPS25H_ERROR or LPS25H_OK.
                 */
                SENSOR_LPS25H_Read_Bytes(LPS25H_SLAVE_ADDRESS, LPS25H_TEMP_OUT_L_ADDR + 0x80, 2, buffer); 
               
                printf("buffer[0]: 0x%02X, buffer[1]: 0x%02X\n", buffer[0], buffer[1]);

                data16Val = (((int16_t)buffer[1]) << 8) + (int16_t)buffer[0];
                 
                calcData = (((float)data16Val/480.0) + 42.5);
                  
                unsigned_data16Val=data16Val;
                printf("data16Val: 0x%4X, Temperature: %fC\n", unsigned_data16Val, calcData);
                
                break;
            }  

            g_print_menu_control_flag = TRUE;

            if('x' == (char)ch)
            {
                break;
            }
        }
    }
    
    return;
}

