/*
 * (C) COPYRIGHT 2009 CRZ
 *
 * File Name : main.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 08/12/2009
 */

/* includes */

#include "hw_config.h"
#include "global_types.h"
#include "uart_queue.h"

/* global variables */

RCC_ClocksTypeDef  rcc_clocks;

/* functions */

void System_Information(void)
{
    printf("IoT AnyThing AT100 Test ... %s %s\n", __DATE__, __TIME__);

    printf("SYSCLK_Frequency = %d\n",rcc_clocks.SYSCLK_Frequency );
    printf("HCLK_Frequency   = %d\n",rcc_clocks.HCLK_Frequency );
    printf("PCLK1_Frequency  = %d\n",rcc_clocks.PCLK1_Frequency );
    printf("PCLK2_Frequency  = %d\n\n",rcc_clocks.PCLK2_Frequency );
}

/**
  * @brief  Configures the used Timers.
  * @param  None
  * @retval None
  */
static void TIM_Configuration(void)
{
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

    /* TIM2 configuration */
    TIM_TimeBaseStructure.TIM_Period = 500 - 1;
    TIM_TimeBaseStructure.TIM_Prescaler = 32000 - 1;
    TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;    
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;  
    TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

    /* TIM2 enable counter */
    TIM_Cmd(TIM2, ENABLE);

    /* Immediate load of TIM2 Precaler value */
    TIM_PrescalerConfig(TIM2, 0xEA5F, TIM_PSCReloadMode_Immediate);

    /* Clear TIM2 update pending flag */
    TIM_ClearFlag(TIM2, TIM_FLAG_Update);

    /* Enable TIM2 Update interrupt */
    TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);
}

void LedControl_forProgramStart(void)
{
#define  LED_CTRL_START_DELAY  30

    LED_On_All();
    Delay(LED_CTRL_START_DELAY);
    LED_Blink_All(5, LED_CTRL_START_DELAY);
    Delay(LED_CTRL_START_DELAY);
    LED_Off_All();
}

bool g_print_menu_control_flag = TRUE;

static void print_menu_command(void)
{
    if(TRUE == g_print_menu_control_flag)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("0> System Information\n");
        printf("Z> ZigBee Reset Pin Open\n");
        printf("T> ZigBee Comm Test\n");
        printf("---------------------\n");
        printf("1> LED Test\n");
        printf("2> MPU 9150 9 Axis Sensor Test\n");
        printf("---------------------\n");
        printf("3> Ambient LightL (BH17) Sensor Test\n");
        printf("4> Temp & Humidity (HTU21) Sensor Test\n");
        printf("---------------------\n");
        printf("5> Actuator Test\n");
        printf("6> Actuator Test - ZigBee Comm TX\n");
//        printf("7> Actuator Test - ZigBee Comm RX\n");
        printf("8> PIR Sensor (AMN4xx1) Test\n");	
        printf("9> Pressure & temp (LPS25H) Sensor Test\n");	
        printf("---------------------\n");
        printf("x> quit\n\n");
    }

    g_print_menu_control_flag = FALSE;
}

static void ZigBee_Initialization(void)
{
    GPIO_SetBits(GPIO_ZIGBEE_RESET, GPIO_ZIGBEE_RESET_PIN);
    Delay(10);
    GPIO_ResetBits(GPIO_ZIGBEE_RESET, GPIO_ZIGBEE_RESET_PIN);
    Delay(10);
    GPIO_SetBits(GPIO_ZIGBEE_RESET, GPIO_ZIGBEE_RESET_PIN);
}

static void ZigBee_Reset_Pin_Open(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    DEBUG_MSG_FUNC_START;

    /* Configure the ZigBee Reset pin */

    GPIO_InitStructure.GPIO_Pin   = GPIO_ZIGBEE_RESET_PIN;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
    GPIO_Init(GPIO_ZIGBEE_RESET, &GPIO_InitStructure);
}

static void ZigBee_Com_Test(void)
{
    uint8_t ch;
    uint8_t send_ch = 'A';

    DEBUG_MSG_FUNC_START;

    while(1)
    {
        if(FALSE == UartQueue_Comm_Is_Empty())
        {
            ch = UartQueue_Comm_DeQueue();
            printf("0x%02X (%c) is received\n", ch, ch);
        }

        UartComm_SendData_OneChar(send_ch);
        printf("%c is sent\n", send_ch);

        Delay(1000);

        if('Z' == send_ch)
        {
            send_ch = 'A';
        }
        else
        {
            send_ch ++;
        }
    }
}

/*
 * Name   : main
 * Input  : None
 * Output : None
 * Return : None
 */

int main(void)
{
    uint8_t ch;

    /* System Clocks Configuration */
    RCC_Configuration();
    RCC_GetClocksFreq(&rcc_clocks);

    /* Configure the GPIO ports */
    GPIO_Configuration();

    LED_Off_All();

    /* NVIC configuration */
    NVIC_Configuration();

    /* EXTI configuration */
    EXTI_Configuration();

    /* UART initialization */
    USART1_Init_for_Debug();
    USART3_Init_for_Comm();

    System_Information();

    /* Configure TIM */
    TIM_Configuration();
    printf("TIM_Configuration() done\n");

    /* Setup SysTick Timer for 1 msec interrupts  */
    if (SysTick_Config(rcc_clocks.SYSCLK_Frequency / 1000))
    { 
        /* Capture error */ 
        while (1);
    }
    printf("SysTick_Config() done\n");

    ZigBee_Initialization();
    printf("ZigBee_Initialization() done\n");

    UartQueue_Comm_Initialize();
    printf("UartQueue_Comm_Initialize() done\n");

    /* Configure I2C */
    I2C_LowLevel_Init();
    printf("I2C_LowLevel_Init() done\n");

    LedControl_forProgramStart();

    Actuator_Port_Initialization();
    printf("Actuator_Port_Initialization() done\n");

#if 0
//    Test_Sensor_AL_BH17();
//    Test_Sensor_HTU21();
#else
    while(1)
    {
        Test_Actuator_ZigBee_Comm_RX();

        if(FALSE == UartQueue_Comm_Is_Empty())
        {
            ch = UartQueue_Comm_DeQueue();
            printf("%c is received\n\n", (char)ch);

            switch((char)ch)
            {
            case '0':
                System_Information();
                break;
            }
        }

        print_menu_command();

        if(FALSE == UartQueue_Serial_Is_Empty())
        {
            ch = UartQueue_Serial_DeQueue();
            printf("%c is selected\n\n", (char)ch);

            switch((char)ch)
            {
            case '0':
                System_Information();
                break;

            case 'Z': case 'z':
                ZigBee_Reset_Pin_Open();
                break;

            case 'T': case 't':
                ZigBee_Com_Test();
                break;

            case '1':
                LED_Test();
                break;

            case '2':
                Test_Sensor_9Axis();
                break;

            case '3':
                Test_Sensor_AL_BH17();
                break;

            case '4':
                Test_Sensor_HTU21();
                break;

            case '5':
                Test_Actuator();
                break;

            case '6':
                Test_Actuator_ZigBee_Comm_TX();
                break;

#if 0
            case '7':
                Test_Actuator_ZigBee_Comm_RX();
                break;
#endif

            case '8':
                Test_Sensor_PIR();
                break;
                
            case '9':
                Test_Sensor_LPS25H();
                break;
            }

            g_print_menu_control_flag = TRUE;

            if('x' == (char)ch)
            {
                break;
            }
        }
    }
#endif

    return 0;
}

